/*
 * Copyright (C) 1993 NeXT Computer, Inc.
 * All Rights Reserved
 */
#ifndef _DOEVENTLOOP_H
#define _DOEVENTLOOP_H

#import <mach/mach.h>
#import <mach/cthreads.h>
#import <remote/NXConnection.h>

/*
 * When a timeout event is requested, a DOTimeoutReceipt is returned.
 * This value can be used to cancel the timeout event.
 */
typedef unsigned long DOTimeoutReceipt;

/*
 * DO timeouts are in milliseconds.
 * A DOTimeInterval represents them
 */
typedef unsigned long DOTimeInterval;

/* 
 * When data is ready on a file descriptor, the handler
 * receives a [dataOnFileDescriptor: handlerData:] message.
 */
@protocol DOFileDescriptorHandling
- (void)dataOnFileDescriptor: (int)fd handlerData: (void *)data;
- (void)fileDescriptorReadyForWriting: (int)fd handlerData: (void *)data;
@end

/* 
 * When a message has been received, the handler
 * receives a [machMessageReceived: handlerData:] message.
 */
@protocol DOMachMessageHandling
- (void)machMessageReceived: (msg_header_t *)msg handlerData: (void *)data;
@end

/* 
 * When a timeout occurs, the handlers receives a
 * [timeoutOccurred: handlerData:] message.
 */
@protocol DOTimeoutHandling
- (void)timeoutOccurred: (DOTimeoutReceipt)receipt handlerData: (void *)data;
@end

#include <remote/doeventloop_private.h>

/*
 * The DOEventLoop class
 */    
@interface DOEventLoop: Object <DOMachMessageHandling>
{
@private
    int have_threads;
    cthread_t child_thread;
    cthread_t main_thread;
    int child_pipe[2];
    DOTimeoutReceipt next_receipt;
    port_t event_port;
#if	   NeXT_PDO
    port_t notify_port;
#endif	/* NeXT_PDO */
    port_set_name_t portset;
    unsigned nfd_handlers;
    _fd_handler_rec_t *fd_handlers;
    unsigned nport_handlers;
    _port_handler_rec_t *port_handlers;
    _timeout_handler_list_t timeout_handlers;
    int stopped;
}

/*
 * Handle port-death notifications synchronously, even in a multi-threaded
 * environment.  This must be called before the first DOEventLoop is
 * instantiated, and it may only be called once.  Subsequent calls will
 * be summarily ignored.
 */
+ (void) setSynchronousPortDeathNotification:(BOOL)flag;

/*
 * Add a file descriptor to the list of file descriptors to check for
 * data (read data only).
 *
 * The data argument is passed back to the handler during callback.
 * 
 * Returns self if the file descriptor was added, nil otherwise.
 */
- addFileDescriptor: (int)fd handler: (id <DOFileDescriptorHandling>)handler 
 handlerData: (void *)data;

- addFileDescriptorForWriting: (int)fd handler: (id <DOFileDescriptorHandling>)handler 
 handlerData: (void *)data;

- addFileDescriptorForReading: (int)fd handler: (id <DOFileDescriptorHandling>)handler 
 handlerData: (void *)data;

- addFileDescriptorForReadingAndWriting: (int)fd handler: (id <DOFileDescriptorHandling>)handler 
 handlerData: (void *)data;

/*
 * Remove the file descriptor from the list
 *
 * Returns nil if the file descriptor is not found, self otherwise.
 */
- removeFileDescriptor: (int)fd;

/*
 * Add a timeout event.
 *
 * The data argument is passed back to the handler during callback.
 * 
 */
- (DOTimeoutReceipt)addTimeoutEntry: (DOTimeInterval) interval
 handler: (id <DOTimeoutHandling>)handler handlerData: (void *)data;

/*
 * Remove a timeout event. Returns self if the entry is found, nil otherwise.
 */
- removeTimeoutEntry: (DOTimeoutReceipt)entry;

/*
 * Add a port which will receive messages
 *
 * The data argument is passed back to the handler during callback.
 * Returns self if the port was successfully added, nil otherwise.
 */
- addPort: (port_t)port handler: (id <DOMachMessageHandling>)handler 
 handlerData: (void *)data;

/*
 * Remove the port. Returns nil if the port is not found, self otherwise.
 */
- removePort: (port_t)port;

/*
 * Add an NXConnection to be listened on
 *
 */
- addConnection: (NXConnection *)connection;

/*
 * Run the event loop. Returns self if [stop] is called or if nil there is a 
 * serious error (should not happen with correct user code). Example of
 * incorrect code is to deallocate a port, and forget to remove it from 
 * the event loop.
 */
- run;

/*
 * Stop the event loop, causing [run] to terminate.
 */
- (void)stop;

- free;

@end

#endif /* _DOEVENTLOOP_H */



